import java.io.*;
import java.awt.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.text.*;

public class ViewFrameChooser extends JFrame
                              implements DocumentListener, ActionListener
{
    private int              Nconnect = 2;
    private String[]         connect_names = { "Disconnected State",
                                               "Connected State" };
    private int              connect_idx = 0;
    private JRadioButton[]   connect_btns;

    private int              Nview;
    private int              view_idx;
    private JRadioButton[]   view_btns;

    private int              Nframe;
    private int              frame_idx;
    private JLabel           frame_idx_tag;
    private JTextField       frame_idx_fld;
    private JButton          prev_btn;
    private JButton          display_btn;
    private JButton          next_btn;

            Mainwin          init_win;
            SLOG_InputStream slog = null;
            SLOG_DirEntry    slog_dir_entry = null;
            SLOG_Frame       slog_frame = null;
            RecordHandler    clog_plotdata = null;

    private FrameDisplay     frame_display = null;


    public ViewFrameChooser( int in_Nframe, int in_Nview, String[] view_names )
    {
        //  Initialization of frame_idx & view_idx
        if ( in_Nframe > 0 ) {
            Nframe    = in_Nframe;
            frame_idx = 0;
        }
        else
            return;

        if ( in_Nview > 0 ) {
            Nview           = in_Nview;
            view_idx        = 0;
        }
        else
            return;

        // top_panel consists 3 subpanels: 
        // view_panel, idx_panel( text field ), btn_panel( buttons )
        setTitle( "View & Frame Chooser" );
        //  setSize( 300, 200 );
        setBackground( Color.gray );
        getContentPane().setLayout( new GridBagLayout() );
        GridBagConstraints constraint = new GridBagConstraints();

        //  Specify connect_panel
        JPanel connect_panel = new JPanel();
        connect_panel.setBorder( new TitledBorder( new EtchedBorder(),
                                                   "Connectivity Options" ) );
        connect_btns = new JRadioButton[ Nconnect ];
        ButtonGroup connect_group = new ButtonGroup();
        for ( int ii = 0; ii < Nconnect; ii++ ) {
            connect_btns[ ii ] = new JRadioButton( connect_names[ ii ] );
            connect_btns[ ii ].addActionListener( this );
            connect_group.add( connect_btns[ ii ] );
            connect_panel.add( connect_btns[ ii ] );
        }
        connect_btns[ connect_idx ].doClick();
        constraint.fill  = GridBagConstraints.HORIZONTAL;
        constraint.gridx = 0; constraint.gridy = 0;
        getContentPane().add( connect_panel, constraint );

        //  Specify view_panel
        JPanel view_panel = new JPanel();
        view_panel.setBorder( new TitledBorder( new EtchedBorder(),
                                                "View Options" ) );
        view_btns = new JRadioButton[ Nview ];
        ButtonGroup view_group = new ButtonGroup();
        for ( int ii = 0; ii < Nview; ii++ ) {
            view_btns[ ii ] = new JRadioButton( view_names[ ii ] );
            view_btns[ ii ].addActionListener( this );
            view_group.add( view_btns[ ii ] );
            view_panel.add( view_btns[ ii ] );
        }
        view_btns[ view_idx ].doClick();
        constraint.fill  = GridBagConstraints.HORIZONTAL;
        constraint.gridx = 0; constraint.gridy = 1;
        getContentPane().add( view_panel, constraint );
        
        //  Specify idx_panel( text field )
        JPanel idx_panel = new JPanel();
        idx_panel.setBorder( new TitledBorder( new EtchedBorder(),
                                               "Frame Information" ) );
        idx_panel.setLayout( new GridBagLayout() );
        constraint.fill  = GridBagConstraints.HORIZONTAL;

            JLabel info_tag = new JLabel( " Number of Frames = " );
            constraint.gridx = 0; constraint.gridy = 0;
            idx_panel.add( info_tag, constraint );

            JLabel info_value = new JLabel( Integer.toString( Nframe ) );
            constraint.gridx = 1; constraint.gridy = 0;
            idx_panel.add( info_value, constraint );

            frame_idx_fld = new JTextField( 10 );
            frame_idx_fld.setHorizontalAlignment( JTextField.CENTER );
            frame_idx_fld.setText( Integer.toString( frame_idx ) );

            frame_idx_tag = new JLabel( " Current Frame        = " );
            frame_idx_tag.setLabelFor( frame_idx_fld );

            constraint.gridx = 0; constraint.gridy = 1;
            idx_panel.add( frame_idx_tag, constraint );
            constraint.gridx = 1; constraint.gridy = 1;
            idx_panel.add( frame_idx_fld, constraint );

        constraint.gridx = 0; constraint.gridy = 2;
        getContentPane().add( idx_panel, constraint );

        //  Specify btn_panel( buttons )
        JPanel btn_panel = new JPanel();
        btn_panel.setBorder( new TitledBorder( new EtchedBorder(),
                                               "Frame Operations" ) );
        btn_panel.setLayout( new FlowLayout() );
        Dimension btn_dim = new Dimension( 90, 50 );

            ImageIcon back_arrow_icon
                      = new ImageIcon( "images/prev_arrow.gif" );
            prev_btn = new JButton( "Previous", back_arrow_icon );
            prev_btn.setVerticalTextPosition( AbstractButton.BOTTOM );
            prev_btn.setHorizontalTextPosition( AbstractButton.CENTER );
            prev_btn.setPreferredSize( btn_dim );
            prev_btn.setToolTipText( "Decrement the frame index counter" );
            prev_btn.setEnabled( false );
            prev_btn.addActionListener( this );
            btn_panel.add( prev_btn ); 

            ImageIcon load_arrow_icon
                      = new ImageIcon( "images/load_arrow.gif" );
            display_btn = new JButton( "Display", load_arrow_icon );
            display_btn.setVerticalTextPosition( AbstractButton.BOTTOM );
            display_btn.setHorizontalTextPosition( AbstractButton.CENTER );
            display_btn.setPreferredSize( btn_dim );
            display_btn.setToolTipText( "Read the requested frame "
                                      + "and Display it" );
            display_btn.setEnabled( false );
            display_btn.addActionListener( this );
            btn_panel.add( display_btn );

            ImageIcon next_arrow_icon
                      = new ImageIcon( "images/next_arrow.gif" );
            next_btn = new JButton( "Next", next_arrow_icon );
            next_btn.setVerticalTextPosition( AbstractButton.BOTTOM );
            next_btn.setHorizontalTextPosition( AbstractButton.CENTER );
            next_btn.setPreferredSize( btn_dim );
            next_btn.setToolTipText( "Increment the frame index counter" );
            next_btn.setEnabled( false );
            next_btn.addActionListener( this );
            btn_panel.add( next_btn ); 

        constraint.fill  = GridBagConstraints.HORIZONTAL;
        constraint.gridx = 0; constraint.gridy = 3;
        getContentPane().add( btn_panel, constraint );

        frame_idx_fld.getDocument().addDocumentListener( this );

        FrameButtonsIdxFldEnabled( frame_idx_fld.getText() );

        addWindowListener( new WindowAdapter()
                               {
                                   public void windowClosing( WindowEvent e )
                                   { 
                                       init_win.enableRead();
                                       init_win.slog = null;
                                       init_win.frame_chooser = null;
                                       free();
                                   }
                               }
                         );
    }

    //  Set Frame Buttons according to the frame_idx and Nframe
    public void EnableFrameButtonsIdxFld()
    {
        frame_idx_fld.setEnabled( true );

        if ( frame_idx >= 0 && frame_idx < Nframe ) {
            if ( frame_idx > 0 )
                prev_btn.setEnabled( true );
            else
                prev_btn.setEnabled( false );

            display_btn.setEnabled( true );

            if ( frame_idx < Nframe-1 )
                next_btn.setEnabled( true );
            else
                next_btn.setEnabled( false );
        }
        else {
            prev_btn.setEnabled( false );
            display_btn.setEnabled( false );
            next_btn.setEnabled( true );
        }
    }

    public void DisableFrameButtonsIdxFld()
    {
        frame_idx_fld.setEnabled( false );

        prev_btn.setEnabled( false );
        display_btn.setEnabled( false );
        next_btn.setEnabled( false );
    }

    private void FrameButtonsIdxFldEnabled( String str )
    {
        if ( str.length() > 0 ) {
            try { 
                frame_idx = Integer.parseInt( str );
                EnableFrameButtonsIdxFld();
                return; 
            }
            catch ( NumberFormatException nerr ) { 
                System.err.println( "Invalid number format" );
            }
        }

        DisableFrameButtonsIdxFld();
    }

    public void changedUpdate( DocumentEvent event )
    {
        FrameButtonsIdxFldEnabled( frame_idx_fld.getText() );
    }

    public void insertUpdate( DocumentEvent event )
    {
        changedUpdate( event );
    }

    public void removeUpdate( DocumentEvent event )
    {
        changedUpdate( event );
    }

    public void actionPerformed( ActionEvent event )
    {
        for ( int ii = 0; ii < Nconnect; ii++ )
            if ( event.getSource() == connect_btns[ ii ] ) {
                connect_idx = ii;
                return;
            }

        for ( int jj = 0; jj < Nview; jj++ )
            if ( event.getSource() == view_btns[ jj ] ) {
                view_idx = jj;
                return;
            }

        if ( event.getSource() == display_btn ) {
            System.out.println( "Frame Index = " + frame_idx + ",  "
                              + "View Action Index = " + view_idx + ",  "
                              + "Connect Action Index = " + connect_idx );
            ReadLogFrame();
            DisplayFrame();
            return;
        }
        if ( event.getSource() == prev_btn ) {
            frame_idx--;
            frame_idx_fld.setText( Integer.toString( frame_idx ) );
            return;
        }
        if ( event.getSource() == next_btn ) {
            frame_idx++;
            frame_idx_fld.setText( Integer.toString( frame_idx ) );
            return;
        }
    }

    public void ReadLogFrame()
    {
        if ( frame_idx >= 0 ) {
            display_btn.setEnabled( false );
            slog_dir_entry = slog.dir.EntryAt( frame_idx );
            try { slog_frame = slog.GetFrame( slog_dir_entry.fptr2framehdr ); }
            catch ( IOException ioerr )
                { System.err.println( "ReadLogFrame() has IO error" ); }
            debug.println( "slog_dir_entry = " + slog_dir_entry );
            clog_plotdata = new RecordHandler( slog, slog_frame );
            display_btn.setEnabled( true );
        }
    }

    private void DisplayFrame()
    {
        if ( clog_plotdata != null ) {
            display_btn.setEnabled( false );
            init_win.waitCursor();
            frame_display = new FrameDisplay( init_win, this, clog_plotdata,
                                              slog_dir_entry.starttime,
                                              slog_dir_entry.endtime );
            init_win.normalCursor();
            /*
            frame_display.addWindowListener( new WindowAdapter()
                {
                    public void windowClosing( WindowEvent e )
                    { 
                        display_btn.setEnabled( true );
                        SetFrameButtonsEnabled( frame_idx_fld.getText() );
                    };
                }
                                           );
            */
        }
    }

    private void free()
    {
        for ( int ii = 0; ii < Nconnect; ii++ )
            connect_btns[ ii ] = null;
        for ( int ii = 0; ii < Nview; ii++ )
            view_btns[ ii ] = null;
        frame_idx_tag  = null;
        frame_idx_fld  = null;
        prev_btn       = null;
        display_btn    = null;
        next_btn       = null;

        slog           = null;
        slog_dir_entry = null;
        slog_frame     = null;
        clog_plotdata  = null;

        frame_display  = null;
    }

    public void SetInitWindow( Mainwin in_initwin )
    {
        init_win = in_initwin;
    }

    public void SetLogFileHdr( SLOG_InputStream in_slog )
    {
        slog = in_slog;
    }

    public int GetFrameIdx()
    {
        return frame_idx;
    }

    public int GetViewActionIdx()
    {
        return view_idx;
    }

    public int GetConnectActionIdx()
    {
        return connect_idx;
    }
}
